//
// © 2005-2008 Marko Lindqvist
//

#include "cell.h"

#include <iostream>

#include "sdksui.h"
#include "sudoku.h"


cell::cell(sudoku *parent_in)
{
  int i;
  int size;

  parent = parent_in;
  size = parent->get_size();

  result = 0;
  possibility_count = size*size;
  possibilities = new bool[size*size + 1];

  for(i = 1 ; i <= size*size; i++) {
    possibilities[i] = true;
  }
}


cell::~cell()
{
  delete [] possibilities;
}


void cell::print()
{
  if(result) {
    std::cout << result;
  } else {
    std::cout << "?" ;
  }
}


bool cell::set_result(int result_in, const char *reason)
{
  int i;
  char logmsg[100];
  int size = parent->get_size();

  if(result) {
    return false;
  }

  sprintf(logmsg, "Solved: %d (%s)", result_in, reason);
  parent->get_ui()->solve_log(this, logmsg);

  result = result_in;
  possibility_count = 1;

  for(i = 0; i <= size*size; i++) {
    if(i != result) {
      possibilities[i] = false;
    }
  }

  return true;
}


int cell::solved()
{
  return result;
}


bool cell::is_possible(int result_in)
{
  return possibilities[result_in];
}


remove_result cell::remove_possibility(int removed, const char *reason)
{
  int i;
  char logmsg[100];

  if(!possibilities[removed]) {
    return RR_KNOWN;
  }

  sprintf(logmsg, "Possibility %d removed (%s)", removed, reason);
  parent->get_ui()->solve_log(this, logmsg);

  possibility_count--;
  possibilities[removed] = false;

  // Solved
  if(possibility_count == 1) {
    // Just iterate until only possibility found
    for(i = 1; ! possibilities[i]; i++) ;
    set_result(i, "Only possibility left");
    return RR_SOLVED;
  }
  return RR_ADVANCE;
}


void cell::setxy(int x_in, int y_in)
{
  x = x_in;
  y = y_in;
}


int cell::get_x()
{
  return x;
}


int cell::get_y()
{
  return y;
}
