//
// © 2005-2008 Marko Lindqvist
//

#include "sdks.h"

#include <stdlib.h>
#include <iostream>

#include "cell.h"
#include "sdksoption.h"
#include "sdksui.h"
#include "sudoku.h"
#include "sdksversion.h"


int main(int argc, char *argv[])
{
  sudoku *si;
  sdks_ui ui;
  int i;
  int base = 3;
  bool params_only = false;

  init_options();

  for (i = 1; i < argc; i++) {
    if (!strncmp(argv[i], PARAM_EXPLAIN_L, strlen(PARAM_EXPLAIN_L)) ||
        !strncmp(argv[i], PARAM_EXPLAIN_S, strlen(PARAM_EXPLAIN_S))) {
      set_option(OPTION_EXPLAIN, true);
    } else if (!strncmp(argv[i], PARAM_HELP_L, strlen(PARAM_HELP_L)) ||
               !strncmp(argv[i], PARAM_HELP_S, strlen(PARAM_HELP_S))) {
      std::cout << "Usage: " << argv[0];
      std::cout << " [" PARAM_SIZE_L      "|" PARAM_SIZE_S      " <value>]";
      std::cout << " [" PARAM_EXPLAIN_L   "|" PARAM_EXPLAIN_S   "]";
      std::cout << " [" PARAM_QUIET_L     "|" PARAM_QUIET_S     "]";
      std::cout << " [" PARAM_INTERMED_L  "|" PARAM_INTERMED_S  "]";
      std::cout << " [" PARAM_SOLVETIME_L "|" PARAM_SOLVETIME_S "]";
      std::cout << " [" PARAM_HELP_L      "|" PARAM_HELP_S      "]";
      std::cout << " [" PARAM_VERSION_L   "|" PARAM_VERSION_S   "]";
      std::cout << "\n\n";
      std::cout << "Enter sudoku line by line.\n";
      std::cout << "(size^2) characters each line, (size^2) lines.\n";
      std::cout << " -> Default size of 3 means 9x9 squares.\n";
      std::cout << "Mark unknown by '?', known by numerical value.\n";
      std::cout << "\nExample:\n";
      std::cout << "3???2?698\n";
      std::cout << "?62?9???4\n";
      std::cout << "4???1??5?\n";
      std::cout << "????82??1\n";
      std::cout << "??6?7?3??\n";
      std::cout << "7??36????\n";
      std::cout << "?4??5???2\n";
      std::cout << "9???3?54?\n";
      std::cout << "185?4???7" << std::endl;

      params_only = true;
    } else if (!strncmp(argv[i], PARAM_VERSION_L, strlen(PARAM_VERSION_L)) ||
               !strncmp(argv[i], PARAM_VERSION_S, strlen(PARAM_VERSION_S))) {
      std::cout << "Version: " SDKS_VERSION << std::endl;
      params_only = true;
    } else if (!strncmp(argv[i], PARAM_QUIET_L, strlen(PARAM_QUIET_L)) ||
               !strncmp(argv[i], PARAM_QUIET_S, strlen(PARAM_QUIET_S))) {
      set_option(OPTION_QUIET, true);
    } else if (!strncmp(argv[i], PARAM_INTERMED_L, strlen(PARAM_INTERMED_L)) ||
               !strncmp(argv[i], PARAM_INTERMED_S, strlen(PARAM_INTERMED_S))) {
      set_option(OPTION_INTERMED, true);
    } else if (!strncmp(argv[i], PARAM_SOLVETIME_L, strlen(PARAM_SOLVETIME_L)) ||
               !strncmp(argv[i], PARAM_SOLVETIME_S, strlen(PARAM_SOLVETIME_S))) {
      set_option(OPTION_SOLVETIME, true);
    } else if (!strncmp(argv[i], PARAM_SIZE_L, strlen(PARAM_SIZE_L)) ||
               !strncmp(argv[i], PARAM_SIZE_S, strlen(PARAM_SIZE_S))) {
      if (argc <= i + 1) {
        std::cerr << "Value for parameter \"" << argv[i] << "\" missing!" << std::endl;
        return EXIT_FAILURE;
      }
      base = atoi(argv[i+1]);

      if (base < MIN_BASE || base > MAX_BASE) {
        std::cerr << "Illegal value " << base << " for \"" << argv[i] << "\".\n";
        std::cerr << "Minimum is " << MIN_BASE << " and maximum " << MAX_BASE << ".";
        std::cerr << std::endl;
        return EXIT_FAILURE;
      }

      i++; // Do not handle value as independent parameter
    } else {
      std::cerr << "Unknown parameter \"" << argv[i] << "\"!" << std::endl;
      return EXIT_FAILURE;
    }
  }

  if (params_only) {
    return EXIT_SUCCESS;
  }

  if (!get_option(OPTION_QUIET)) {
    std::cout << "sdks " SDKS_VERSION "\n";
    std::cout << "Try '" << argv[0] << " --help' for usage info.\n";
    std::cout << "Waiting input (ctrl^C to abort)" << std::endl;
  }

  si = new sudoku(base, &ui);

  if(! si->read()) {
    return EXIT_FAILURE;
  }
  std::cout << "Starting situation:" << std::endl;

  if (get_option(OPTION_INTERMED)) {
    ui.print_sudoku_possibilities(si);
  } else {
    ui.print_sudoku(si);
  }
  si->solve();
  if (si->check_legality()) {
    // Legal solution
    std::cout << "Result:" << std::endl;
    ui.print_sudoku(si);
  } else {
    std::cout << "Got illegal result" << std::endl;
    if (get_option(OPTION_INTERMED)) {
      ui.print_sudoku(si);
    }
  }

  delete si;

  return EXIT_SUCCESS;
}
